#  Copyright (C) 1999-2010
#  Smithsonian Astrophysical Observatory, Cambridge, MA, USA
#  For conditions of distribution and use, see copyright notice in "copyright"

package provide DS9 1.0

proc CubeDef {} {
    global cube

    set cube(top) .cube
    set cube(mb) .cubemb

    set cube(axis) 2
    set cube(id) 0
}

proc CubeStop {} {
    global cube

    if {$cube(id)>0} {
	after cancel $cube(id)
	set cube(id) 0
    }
}

proc CubePlay {} {
    global cube

    if {$cube(id) == 0} {
	CubeTimer
    }
}

proc CubeTimer {} {
    global cube
    global cubedlg
    global current
    global blink
    global rgb

    if {$current(frame) != {}} {
	set slice [$current(frame) get fits slice $cube(axis)]
	set depth [$current(frame) get fits depth $cube(axis)]

	if {$slice == $depth} {
	    set slice 1
	} else {
	    set slice [expr $slice+1]
	}

	RGBEvalLock rgb(lock,slice) \
	    "$current(frame) update fits slice $cube(axis) $slice"

	set cubedlg(slice,$cube(axis)) $slice
	ContourUpdateNow
	UpdateMain
    }

    set cube(id) [after $blink(interval) CubeTimer]
}

proc CubeFirst {} {
    global cube
    global cubedlg
    global current
    global rgb

    CubeStop

    set slice 1
    if {$current(frame) != {}} {
	puts stderr "CubeFirst $current(frame) $cube(axis) $slice"
	RGBEvalLock rgb(lock,slice) \
	    "$current(frame) update fits slice $cube(axis) $slice"

	set cubedlg(slice,$cube(axis)) $slice
	ContourUpdateNow
	UpdateMain
    }
}

proc CubePrev {} {
    global cube
    global cubedlg
    global current
    global rgb

    CubeStop

    if {$current(frame) != {}} {
	set slice [$current(frame) get fits slice $cube(axis)]
	set depth [$current(frame) get fits depth $cube(axis)]

	if {$slice == 1} {
	    set slice $depth
	} else {
	    set slice [expr $slice-1]
	}

	RGBEvalLock rgb(lock,slice) \
	    "$current(frame) update fits slice $cube(axis) $slice"

	set cubedlg(slice,$cube(axis)) $slice
	ContourUpdateNow
	UpdateMain
    }
}

proc CubeNext {} {
    global cube
    global cubedlg
    global current
    global rgb

    CubeStop

    if {$current(frame) != {}} {
	set slice [$current(frame) get fits slice $cube(axis)]
	set depth [$current(frame) get fits depth $cube(axis)]

	if {$slice == $depth} {
	    set slice 1
	} else {
	    set slice [expr $slice+1]
	}

	RGBEvalLock rgb(lock,slice) \
	    "$current(frame) update fits slice $cube(axis) $slice"

	set cubedlg(slice,$cube(axis)) $slice
	ContourUpdateNow
	UpdateMain
    }
}

proc CubeLast {} {
    global cube
    global cubedlg
    global current
    global rgb

    CubeStop

    if {$current(frame) != {}} {
	set slice [$current(frame) get fits depth $cube(axis)]
	RGBEvalLock rgb(lock,slice) \
	    "$current(frame) update fits slice $cube(axis) $slice"

	set cubedlg(slice,$cube(axis)) $slice
	ContourUpdateNow
	UpdateMain
    }
}

proc CubeApply {ii slice} {
    global cubedlg
    global current
    global rgb

    CubeStop

    if {$current(frame) != {}} {
	RGBEvalLock rgb(lock,slice) \
	    "$current(frame) update fits slice $ii $slice"

	set cubedlg(slice,$ii) $slice
	ContourUpdateNow
	UpdateMain
    }
}

proc CubeDialog {} {
    global cube
    global cubedlg
    global current
    global ds9
    global blink

    # see if we already have a window visible
    if [winfo exists $cube(top)] {
	raise $cube(top)
	return
    }

    set w $cube(top)
    set title [msgcat::mc {Data Cube}]

    # create the cube window

    toplevel $w
    switch $ds9(wm) {
	x11 -
	win32 {}
	aqua {::tk::unsupported::MacWindowStyle style $w document "closeBox collapseBox"}
    }

    wm title $w $title
    wm iconname $w $title
    wm group $w $ds9(top)
    wm protocol $w WM_DELETE_WINDOW CubeDestroyDialog

    $w configure -menu $cube(mb)

    menu $cube(mb)

    AppleMenu cube(mb)

    $cube(mb) add cascade -label [msgcat::mc {File}] -menu $cube(mb).file
    $cube(mb) add cascade -label [msgcat::mc {Interval}] -menu $cube(mb).blink

    menu $cube(mb).file
    $cube(mb).file add command -label [msgcat::mc {Close}] \
	-command CubeDestroyDialog

    menu $cube(mb).blink
    $cube(mb).blink add radiobutton -label ".125 [msgcat::mc {Seconds}]" \
	-variable blink(interval) -value 125
    $cube(mb).blink add radiobutton -label ".25 [msgcat::mc {Seconds}]" \
	-variable blink(interval) -value 250
    $cube(mb).blink add radiobutton -label ".5 [msgcat::mc {Seconds}]" \
	-variable blink(interval) -value 500
    $cube(mb).blink add radiobutton -label "1 [msgcat::mc {Seconds}]" \
	-variable blink(interval) -value 1000
    $cube(mb).blink add radiobutton -label "2  [msgcat::mc {Seconds}]" \
	-variable blink(interval) -value 2000
    $cube(mb).blink add radiobutton -label "4 [msgcat::mc {Seconds}]" \
	-variable blink(interval) -value 4000
    $cube(mb).blink add radiobutton -label "8 [msgcat::mc {Seconds}]" \
	-variable blink(interval) -value 8000

    frame $w.param -relief ridge -bd 1
    frame $w.vcr -relief ridge -bd 1
    pack $w.param $w.vcr -fill x -expand true -ipadx 4 -ipady 4

    set cubedlg(title) [label $w.param.title -text [msgcat::mc {Axis}]]
    set cubedlg(title2) [label $w.param.title2 -text [msgcat::mc {Slice}]]
    for {set ii 2} {$ii<$ds9(FTY_MAXAXES)} {incr ii} {
	set cubedlg(scale,$ii) [scale $w.param.slice$ii \
				    -from 1 -to 1 -length 300 \
				    -variable cubedlg(slice,$ii) \
				    -orient horizontal \
				    -tickinterval 1 \
				    -showvalue true -resolution 1 \
				    -command [list CubeApply $ii]]
	set cubedlg(chk,$ii) [radiobutton $w.param.chk$ii \
				  -text [expr $ii+1] \
				  -variable cube(axis) \
				  -value $ii]
    }

    button $w.vcr.first -text [msgcat::mc {First}] -command CubeFirst
    button $w.vcr.prev -text [msgcat::mc {Previous}] -command CubePrev
    button $w.vcr.stop -text [msgcat::mc {Stop}] -command CubeStop
    button $w.vcr.play -text [msgcat::mc {Play}] -command CubePlay
    button $w.vcr.next -text [msgcat::mc {Next}] -command CubeNext
    button $w.vcr.last -text [msgcat::mc {Last}] -command CubeLast

    pack $w.vcr.first $w.vcr.prev $w.vcr.stop $w.vcr.play \
	$w.vcr.next $w.vcr.last -side left -expand true

    UpdateCubeDialog
}

proc CubeDestroyDialog {} {
    global cube
    global cubedlg

    if {[winfo exists $cube(top)]} {
	destroy $cube(top)
	destroy $cube(mb)
    }

    if {[info exists cubedlg]} {
	unset cubedlg
    }
}

proc UpdateCubeDialog {} {
    global cube
    global cubedlg
    global current
    global ds9

    global debug
    if {$debug(tcl,update)} {
	puts stderr "UpdateCubeDialog"
    }

    CubeStop

    if {![winfo exists $cube(top)]} {
	return
    }

    for {set ii 2} {$ii<$ds9(FTY_MAXAXES)} {incr ii} {
	if {$current(frame) != {}} {
	    set cubedlg(depth,$ii) [$current(frame) get fits depth $ii]
	} else {
	    set cubedlg(depth,$ii) 1
	}
    }
    
    for {set jj [expr $ds9(FTY_MAXAXES)-1]} {$jj>2} {incr jj -1} {
	if {$cubedlg(depth,$jj) > 1} {
	    break;
	}
    }
    
    # forget everything
    grid forget $cubedlg(title) $cubedlg(title2)
    for {set ii 2} {$ii<$ds9(FTY_MAXAXES)} {incr ii} {
	grid forget $cubedlg(chk,$ii) $cubedlg(scale,$ii)
    }

    if {$jj == 2} {
	# special case
	grid $cubedlg(scale,2) -padx 10
    } else {
	# show it
	grid $cubedlg(title) $cubedlg(title2) -padx 4 -sticky ew

	for {set ii 2} {$ii<=$jj} {incr ii} {
	    grid $cubedlg(chk,$ii) $cubedlg(scale,$ii) -padx 10
	}
    }

    # set intervals
    for {set ii 2} {$ii<=$jj} {incr ii} {
	if {$cubedlg(depth,$ii) < 20} {
	    set interval 1
	} elseif {$cubedlg(depth,$ii) < 100} {
	    set interval 10
	} elseif {$cubedlg(depth,$ii) < 200} {
	    set interval 20
	} else {
	    set interval 50
	}

	$cubedlg(scale,$ii) configure -to $cubedlg(depth,$ii) \
	    -tickinterval $interval
    }

    # reset cube(axis) if needed
    if {$cube(axis) > $jj} {
	set cube(axis) $jj
    }

    # we must do this after the scale has been configured
    for {set ii 2} {$ii<$ds9(FTY_MAXAXES)} {incr ii} {
	if {$current(frame) != {}} {
	    set cubedlg(slice,$ii) [$current(frame) get fits slice $ii]
	} else {
	    set cubedlg(slice,$ii) 1
	}
    }
}

# Process Cmds

proc ProcessDataCubeCmd {varname iname} {
    upvar $varname var
    upvar $iname i

    global cube
    global blink
    global rgb

    CubeDialog

    switch -- [string tolower [lindex $var $i]] {
	play {CubePlay}
	stop {CubeStop}
	next {CubeNext}
	prev {CubePrev}
	first {CubeFirst}
	last {CubeLast}
	interval {incr i; set blink(interval) [expr int([lindex $var $i]*1000)]}
	axis {incr i; set cube(axis) [lindex $var $i]}
	default {CubeApply $cube(axis) [lindex $var $i]}
    }
}

proc ProcessSendDataCubeCmd {proc id param} {
    global current
    global cube
    global blink

    switch -- [string tolower [lindex $param 0]] {
	interval {$proc $id "[expr $blink(interval)/1000.]\n"}
	axis {$proc $id "$cube(axis)\n"}
	default {
	    if {$current(frame) != {}} {
		$proc $id "[$current(frame) get fits slice $cube(axis)]\n"
	    } else {
		$proc $id "1\n"
	    }
	}
    }
}
